# Formulator - Let's you manage Forms on the ZOPE Application Server
# Copyright (C) 2001 Christian Theune, gocept gmbh & co. kg
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Module for Button-Widgets
#
# $Id: HTMLButton.py,v 1.7 2001/06/11 22:12:42 ctheune Exp $

__doc__ = """HTMLWidget Button module."""
__version__ = '0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
from OFS.DTMLDocument import DTMLDocument
import Acquisition
import AccessControl

True=1
False=0

manage_addHTMLButtonForm=HTMLFile('dtml/HTMLButtonAdd', globals())

def manage_addHTMLButton(self, id, REQUEST=None):
    """Add a new HTML Button to the Folder."""
    self._setObject(id, HTMLButton(id, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class HTMLButton(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, \
			AccessControl.Role.RoleManager):
    """HTMLWidget for Buttons"""

    meta_type='HTML Button'
    css_class=None

    colspan=0
    title=hint='&nbsp;'
    
    manage_properties=HTMLFile('dtml/HTMLButtonEdit', globals())
    
    manage_options=(
        {'label':'Properties', 'action':'manage_properties', 'target':None},
        {'label':'Security', 'action':'manage_access', 'target':None},
        )
	
    def __init__(self, id, REQUEST=None):
        self.id=id
	if REQUEST is not None:
	    self.changeProperties(REQUEST)

    def manage_edit(self, REQUEST=None):
        """Save edited properties."""
        self.changeProperties(REQUEST)
        return MessageDialog(title='Edited', message= \
		"Properties for %s changed." % self.id, \
            	action='./manage_main')

    def index_html(self, REQUEST=None):
        if self.type=='submit' and self.isimg:
            type='image'
            imgsrc='src="%s"' % self.imgsrc
        else:
            type=self.type
            imgsrc=''
        return '<input type="%s" name="%s" value="%s" %s>' % \
		(type, self.id, self.value, imgsrc)

    def __str__(self):
        return self.index_html(self)

    def title_or_id(self):
    	return self.title

    def __validate__(self, against=None):
        return True

    def height(self):
        return 1
    
    def changeProperties(self, REQUEST=None):
        if REQUEST is not None:
            self.type=REQUEST.type
            self.value=REQUEST.value
            self.column=REQUEST.column
            self.row=REQUEST.row
	    self.colspan=REQUEST.colspan
            if REQUEST.has_key('isimg'):
                self.isimg=1
            else:
                self.isimg=0
            self.imgsrc = REQUEST.imgsrc
	    
# $Log: HTMLButton.py,v $
# Revision 1.7  2001/06/11 22:12:42  ctheune
# added new changeProperties behaviour for editing Items
# cleaned up
# added support for spanning
# fixed minor bugs / possible bugs
#
# Revision 1.6  2001/06/07 16:26:27  ctheune
# fixed bug 118
#
# Revision 1.5  2001/04/16 17:43:19  ctheune
# Fixed Bug #25, added title Functions
#
# Revision 1.4  2001/04/03 17:45:09  ctheune
# changed cvs tag Revision to ID as described in Bug #10 on bugzilla.gocept.com
#
# Revision 1.3  2001/04/01 15:12:23  ctheune
# fixed Syntax-Bug in line #66
#
# Revision 1.2  2001/04/01 14:39:57  ctheune
# cleaned up code
#
# Revision 1.1  2001/03/30 12:46:22  ctheune
# added HTML button class
#
