# Formulator - Let's you manage Forms on the ZOPE Application Server
# Copyright (C) 2001 Christian Theune, gocept gmbh & co. kg
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Product for Forms
#
# $Id: HTMLForm.py,v 1.33 2001/06/24 07:24:36 ctheune Exp $

__doc__="""HTMLForm product module"""
__version__='0.1'

from Globals import HTMLFile
from Globals import HTML
from Globals import MessageDialog
import Persistence

from OFS.Folder import Folder
from OFS.DTMLDocument import DTMLDocument
import Acquisition
import AccessControl
import types, copy
import HTMLButton

True=1
False=0

valid_meta_types=["HTML Textfield", "HTML Button", "HTML Textarea", \
		    "HTML Menu", "HTML Label", "HTML Fileupload"]

manage_addHTMLFormForm=HTMLFile('dtml/HTMLFormAdd', globals())

def manage_addHTMLForm(self, id, REQUEST=None):
    """Adds a new HTML Form to the ZODB"""
    self._setObject(id, HTMLForm(id, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class HTMLForm(Folder, Persistence.Persistent, Acquisition.Implicit, \
                AccessControl.Role.RoleManager):
    """HTMLForm is a Folder for Widgets"""

    meta_type='HTMLForm'
    dontAllowCopyAndPaste=0
    
    transferMethod='POST'
    spacingcell='&nbsp;'

    cellpadding=2
    cellspacing=0
    
    hiddenValues=()
    
    manage_properties=HTMLFile('dtml/HTMLFormEdit', globals())
    manage_hidden=HTMLFile('dtml/HTMLFormHiddenEdit', globals())
    
    manage_options=(
        {'label':'Contents', 'action':'manage_main', 'target':None},
        {'label':'Properties', 'action':'manage_properties', 'target':None},
	{'label':'Hidden Values', 'action':'manage_hidden', 'target':None},
        {'label':'View', 'action':'index_html', 'target':None},
        {'label':'Security', 'action':'manage_access', 'target':None}
        )

    def __init__(self, id, REQUEST=None):
        self.id=id
	if REQUEST is not None:
	    self.changeProperties(REQUEST)

    def index_html(self, client=None, REQUEST=None):
        """ Renders the complete Form and 
	    returns the corresponding HTML code """

	# I don't know why REQUEST is sometimes available by parameter and 
	# sometimes only reachable through acquisition
        if REQUEST is None:
            if self.REQUEST is not None:
                REQUEST=self.REQUEST

	REQUEST.set('__source_data__', {}) #remove faked source data

	# Get the data to fill the form with.
        dataSource=None
        if self.sourceMethod:
            dataSource=self.aq_acquire(self.sourceMethod)
        if dataSource is not None:
            REQUEST['__source_data__']= \
		self.aq_acquire(self.sourceMethod)(client=self, REQUEST=REQUEST)
	
	# By the example of HTMLMenu.py, this should work without this fix.
	# Don't know why it didn't work before, possible zope change?
	# I will remove these lines when I'm sure.
	if REQUEST.has_key('__source_data__'):
	    if type(REQUEST['__source_data__']) == types.DictType:
		pass 	# DON'T REMOVE IT ... THIS MEANS A DICTIONARY STAYS AS IT IS
		        # REMOVING THIS IF BLOCK WILL LEAD TO the ELSE block ... 
	    elif type(REQUEST['__source_data__']) == types.InstanceType:
		# This hints to a SQL resultset
		try:
			qryResult=REQUEST['__source_data__']
			names=qryResult.names();
			resultdict={}
			for name in names:
				resultdict.update({name:qryResult[0][name]})
			REQUEST['__source_data__']=resultdict
		except:
			REQUEST['__source_data__']=\
				REQUEST['__source_data__'].data_dictionary()
	    else:
		REQUEST['__source_data__']={}

	# STEP 1:
	# intermatrix contains a list of all widgets
	# and the maximum row and column number is detected
        intermatrix=[]
        maxrow=maxcolumn=1
        for widget in self._objects:
            if widget['meta_type'] in valid_meta_types:
                _widget=self.aq_acquire(widget['id'])
                intermatrix.append(_widget)
                if _widget.column > maxcolumn:
                    maxcolumn=_widget.column
                if _widget.row > maxrow:
                    maxrow=_widget.row

	# STEP 2:
        # Move the widgets into an 2D Matrix, 
	# so you can access them via their coordinates
	intermatrix1=[]
        for y in range(int(maxrow)):
            line=[]
            for x in range(int(maxcolumn)):
                line.append(None)
            intermatrix1.append(line)
        for widget in intermatrix:
            intermatrix1[widget.row-1][widget.column-1]=widget

	# STEP 3:
        # Create a list of maximum height usages in one row 
	# (widgetheight+1 for the hint)
        rowheights=[]
        for row in intermatrix1:
            highest=1
            for widget in row:
                if widget is not None:
                    if (widget.height()+1) > highest:
                        highest=(widget.height()+1)
            rowheights.append(highest)

	# STEP 4:
        # Create the product matrix which will contain all physical
	# cells and their respective widgets.
        productmatrix=[]
        arow=[]
        for column in range(maxcolumn*3):        # Every widget uses three cols
            arow.append({'type':'space'})
        for height in rowheights:
            for row in range(height):             # This row is 'height' tall
                productmatrix.append(copy.deepcopy(arow))

        # STEP 5:
	# Finish the product matrix, by feeding the widgets to it.
        rowbase=0
        for row in intermatrix1:
            columnbase=0
            maxheight=1
            for widget in row:
                if widget is not None:
		    # HINT: This should be done otherwise, by using automatic
		    # spanning of the VIRTUAL grid.
		    # I will do this if I have some time for a version 2
		    spanwidth=0
		    if hasattr(widget,'colspan'):
			maxspan=(maxcolumn*3)-columnbase-1
			if widget.colspan>1:
			    spanwidth=(widget.colspan*3)-columnbase-1
			if (widget.colspan==-1) or (spanwidth > maxspan):
			    spanwidth=maxspan
			if (widget.colspan in [0,1]) and widget.height()>1:
			    spanwidth=1
			spanheight=widget.height()
			if spanheight<1:
			    spanheight=1
			for y in range(spanheight):
			    for x in range(spanwidth):
			        productmatrix[rowbase+y][columnbase+x+1].update(\
				    {'type':'span'})
                    productmatrix[rowbase][columnbase].update( \
			    {'type':'title', 'widget':widget},)
            	    productmatrix[rowbase][columnbase+1].update( \
			    {'type':'widget', 'widget':widget, \
			     'colspan':spanwidth})
                    productmatrix[rowbase+widget.height()][columnbase+1].\
		    	    update({'type':'hint', 'widget':widget})
                    if (widget.height()+1) > maxheight:
                	maxheight=(widget.height()+1)
                columnbase=columnbase+3
            rowbase=rowbase+maxheight

	# STEP 6:
	# The generation of HTML begins from here.
	if self.transferMethod=="POST":
		enctype="multipart/form-data"
	else:
		enctype="application/x-www-form-urlencoded"

        htmltext='<form name="%s" action="%s" method="%s" enctype="%s">\n' % \
	    (self.id, self.getAction()+'/'+self.id+'/checkInput', \
		self.transferMethod, enctype)

	if hasattr(self, 'hiddenValues'):
	    #Following IF migrates from some older versions. Should be removed.
	    if type(self.hiddenValues)==types.DictionaryType:
		self.oldHidden=copy.deepcopy(self.hiddenValues)
		newHidden=()
		for oldItem in self.hiddenValues.keys():
		    newHidden=newHidden+((oldItem, \
			self.hiddenValues[oldItem][0], \
			self.hiddenValues[oldItem][1]), )
		self.hiddenValues=copy.deepcopy(newHidden)
		del(newHidden)

	    for hiddenfield in self.hiddenValues:
		value=hiddenfield[1]
  		if REQUEST.has_key('__source_data__'):
		    if REQUEST['__source_data__'].has_key(hiddenfield[0]):
			value=REQUEST['__source_data__'][hiddenfield[0]]
		hiddenhtml='<input type="hidden" name="%s:%s" value="%s">\n'%\
			(hiddenfield[0], hiddenfield[2], value)
		htmltext=htmltext+hiddenhtml

        htmltext=htmltext+'<input type="hidden" name="__transfer_check__" ' \
			    'value="transfer">\n'
	
	htmltext=htmltext+'<table class="form-table" cellspacing="%s"' \
			    'cellpadding="%s">\n' % \
			    (self.cellspacing, self.cellpadding)

        for row in productmatrix:
            htmltext=htmltext+'<tr class="form-row">\n'
            for cell in row:
		css = "form-"
		try:
		    a=REQUEST['error_info'][cell['widget'].id]
		    del(a)
		    css = css + "error-"
		except KeyError:
		    pass
                if cell['type']=='space':
                    htmltext=htmltext+'<td class="%sspacingcell">%s</td>\n' \
					% (css,self.spacingcell)
                elif cell['type']=='title':
                    htmltext=htmltext+'<td class="%stitlecell">%s</td>\n' \
					% (css,cell['widget'].title)
                elif cell['type']=='widget':
                    if cell['widget'].css_class is not None:
                        css_class=css+cell['widget'].css_class
                    else:
                        css_class=css+'widgetcell'
		    rowspan=colspan=''
                    if cell['widget'].height()>1:
                        rowspan='rowspan="%s"' % cell['widget'].height()
		    if cell['colspan']>0:
			colspan='colspan="%s"' % (cell['colspan'])
		    span=rowspan+' '+colspan
                    htmltext=htmltext+'<td class="%s" %s>%s</td>\n' \
					% (css_class, span, cell['widget']. \
					    index_html(self.REQUEST))
                elif cell['type']=='hint':
		    try:
			hint=REQUEST['error_info'][cell['widget'].id]
		    except KeyError:
			hint=self.spacingcell
                    htmltext=htmltext+'<td class="%shintcell">%s</td>\n' \
					% (css,hint)
                elif cell['type']=='span':
                    continue
            htmltext=htmltext+'</tr>\n'
        htmltext=htmltext+'</table>\n</form>'
        return htmltext
    
    __str__=index_html	# wrappers for beeing called by <dtml-var> or thoose
    __call__=index_html
       
    def manage_edit(self, REQUEST=None):
        """Sets new properties"""
	if REQUEST is not None:
	    self.changeProperties(REQUEST)
            return MessageDialog(title='Edited', \
		message='Properties for %s changed.' % self.id, \
		action='./manage_properties')

    def manage_edithidden(self, REQUEST=None):
    	"""Sets new properties for the hidden fields"""
	if not hasattr(self,'hiddenValues'):
	    self.hiddenValues=()
	if REQUEST is not None:
	    if REQUEST.submit=='add':
		self.hiddenValues=self.hiddenValues+( \
		    (REQUEST.id, REQUEST.value, REQUEST.type),)
		return MessageDialog(title='Added', \
		    message='Added new hidden value called %s.' % REQUEST.id, \
		    action='./manage_hidden')
	    elif REQUEST.submit=='edit':
		self.hiddenValues=()
		i=0
		for id in REQUEST.hidden_ids:
		    if id!='':
			self.hiddenValues=self.hiddenValues+ \
			    ((REQUEST.hidden_ids[i], REQUEST.hidden_values[i],\
			      REQUEST.hidden_types[i]), )
		    i+=1
		return MessageDialog(title='Edited', \
		    message='Edited hidden values.', \
		    action='./manage_hidden')

    def changeProperties(self, REQUEST=None):
        if REQUEST is not None:
            self.transferMethod=REQUEST.transferMethod
	    self.callmethod=REQUEST.callmethod
	    self.target=REQUEST.target
	    self.sourceMethod=REQUEST.sourceMethod
	    self.spacingcell=REQUEST.spacingcell
	    self.cellpadding=REQUEST.cellpadding
	    self.cellspacing=REQUEST.cellspacing
    
    def getAction(self):
	"""Finds out, where this form should be submitted to."""
	try:
	    if self.REQUEST['__transfer_check__']:
        	return self.REQUEST['URL2']
	except KeyError:
	    pass
        return self.REQUEST['URL1']

    def title_or_id(self):
        return self.aq_parent.title
    
    def checkInput(self, client=None, REQUEST=None):
        """Checks the input for format mistakes etc."""
        REQUEST['error_info']={'_':[], }
	
	if REQUEST is None:
	    return self.index_html_file(self)
	
	if not REQUEST.has_key("__transfer_check__"):
	    return self.index_html_file(self)

        for Obj in self._objects:
            if not Obj['meta_type'] in valid_meta_types:
		continue
            testObj=self._getOb(Obj['id'])
            if REQUEST.has_key(testObj.id):
                checkValue=REQUEST[testObj.id]
            else:
                checkValue=None
	
            if testObj.__validate__(checkValue) is False:
                REQUEST['error_info'].update({testObj.id:testObj.hint})

        if len(REQUEST['error_info']) > 1:
            return self.aq_parent.index_html(client=self, REQUEST=REQUEST)

        if self.callmethod!='':
            callMethod=self.aq_acquire(self.callmethod)
            result=callMethod(client=self, REQUEST=REQUEST)
	
	if self.target != "":     
    	    REQUEST.RESPONSE.redirect(REQUEST.BASE0+self.target)
	
# $Log: HTMLForm.py,v $
# Revision 1.33  2001/06/24 07:24:36  ctheune
# removed new metadata
#
# Revision 1.32  2001/06/24 07:21:46  ctheune
# added some metadata
#
# Revision 1.31  2001/06/23 16:05:43  ctheune
# fixed css name convention again.
#
# Revision 1.30  2001/06/12 19:36:48  ctheune
# fixed bug #123: retrieving of initial data was not fully functional
#
# Revision 1.29  2001/06/11 22:24:21  ctheune
# removed debugging code
#
# Revision 1.28  2001/06/11 22:13:55  ctheune
# added new changeProperties behaviour for editing Items
# cleaned up
# added support for new spanning Method
# fixed minor bugs / possible bugs
# removed special treatment of Labels
#
# Revision 1.27  2001/06/07 16:26:27  ctheune
# fixed bug 118
#
# Revision 1.26  2001/06/07 15:49:02  ctheune
# fixed minor coding mistakes
#
# Revision 1.25  2001/06/07 15:35:48  ctheune
# fixed bug 107
#
# Revision 1.24  2001/06/07 13:02:55  ctheune
# fixed some syntax errors
# fixed support for HTMLFileupload
# added enctype
#
# Revision 1.23  2001/06/07 11:17:37  ctheune
# added support for file upload classapplication/HTMLForm/HTMLForm.py
#
# Revision 1.22  2001/06/07 16:47:08  ctheune
# not making a redirect if no target is given anymore
#
# Revision 1.21  2001/06/07 16:11:30  ctheune
# minor changes (style)
#
# Revision 1.20  2001/06/05 13:47:39  zagy
# Restored interfaces to data source method. Undocumented interface chances
# are the worst thing I can imagine.
#
# Revision 1.19  2001/06/05 07:35:43  ctheune
# fixed some interfaces (made them correctly beeing implemented)
# and readded the workaround for the dictionary from sql results
#
# Revision 1.18  2001/06/04 21:56:00  ctheune
# fixed bug #65
#
# Revision 1.17  2001/06/04 09:38:07  ctheune
#  fixed bug #99
#  fixed bug #36
#  wrote better comments
#  cleaned up source code & reformatting
#  removed debugging backdoors
#
# Revision 1.16  2001/05/27 10:23:20  ctheune
# fixed bug #37
# fixed bug #57
#
# Revision 1.15  2001/05/10 20:53:55  ctheune
# next fix ...
#
# Revision 1.14  2001/05/10 20:49:54  ctheune
# another midnight fix for migration
#
# Revision 1.13  2001/05/10 20:30:42  ctheune
# implemented transparent migration function for hidden fields
#
# Revision 1.12  2001/05/10 05:01:18  ctheune
# fixed hidden fields again. (used tuples instead of dictionaries)
#
# Revision 1.11  2001/05/09 07:17:01  ctheune
# fixed hidden fields
#
# Revision 1.10  2001/05/07 18:33:42  ctheune
# fixed bug 51
#
# Revision 1.9  2001/05/05 15:57:48  ctheune
# fixed forwarding
#
# Revision 1.8  2001/04/30 15:17:36  ctheune
# added hidden value support
# fixed bug #22
#
# Revision 1.7  2001/04/24 15:30:43  zagy
# %  dontallowcopyandpaste: set to 0
#
# Revision 1.6  2001/04/14 14:56:45  zagy
# Solved: [Bug 20] HTMLForm hinzufuegen: There is no variable named 'HTMLButton'
#
# Revision 1.5  2001/04/12 20:46:37  ctheune
# fixed bug #16
#
# Revision 1.4  2001/04/10 20:00:24  ctheune
# modified for usage with HTML Label
#
# Revision 1.3  2001/04/03 17:45:09  ctheune
# changed cvs tag Revision to ID as described in Bug #10 on bugzilla.gocept.com
#
# Revision 1.2  2001/04/01 14:40:12  ctheune
# cleaned up code
#
# Revision 1.1  2001/03/30 12:46:36  ctheune
# added HTML form class
#
