# Formulator - Let's you manage Forms on the ZOPE Application Server
# Copyright (C) 2001 Christian Theune, gocept gmbh & co. kg
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Module for Menu-Widgets
#
# $Id: HTMLMenu.py,v 1.17 2001/06/27 10:14:06 ctheune Exp $

__doc__="""HTMLWidget Menu module."""
__version__='0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import rexec
import OFS.SimpleItem
from OFS.DTMLDocument import DTMLDocument
import Acquisition
import AccessControl

import re
import types

True=1
False=0

manage_addHTMLMenuForm=HTMLFile('dtml/HTMLMenuAdd', globals())

def manage_addHTMLMenu(self, id, REQUEST=None):
	"""Adds a HTML menu to the folder."""
	self._setObject(id, HTMLMenu(id, REQUEST))
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)

class HTMLMenu(OFS.SimpleItem.Item, Persistent,	Acquisition.Implicit, \
		AccessControl.Role.RoleManager):
	"""HTMLWidget for Menues"""
	datatype='string'

	meta_type='HTML Menu'
	css_class='menu'

        allowzero=0
        zeroform='0'
        zerodata='0'
	required=1
	ismenu=1
	buttonColumns=3
	colspan=0
	
	manage_properties=HTMLFile('dtml/HTMLMenuEdit', globals())
	
	manage_options=(
	    {'label':'Properties', 'action':'manage_properties', 'target':None},
	    {'label':'Security', 'action':'manage_access', 'target':None},
	    )

	def __init__(self, id, REQUEST=None):
		self.id=id
		if REQUEST is not None:
		    self.changeProperties(REQUEST)

	def index_html(self, REQUEST=None):
		if self.ismenu:
			return self.renderMenu(REQUEST)
		else:
			return self.renderButtons(REQUEST)

	def manage_edit(self, REQUEST=None):
		"""Sets the new properties."""
		if REQUEST is not None:
			self.changeProperties(REQUEST)				
			return MessageDialog(title='Edited', \
				message="Properties for %s changed."%self.id, \
				action='./manage_properties')
	
	def changeProperties(self, REQUEST=None):
		if REQUEST is not None:
			self.title=REQUEST.title
			self.hint=REQUEST.hint

			self.column=REQUEST.column
			self.row=REQUEST.row
			self.colspan=REQUEST.colspan

			self.default=REQUEST.default
			self.datatype=REQUEST.datatype
			self.source=REQUEST.source

			self.size=REQUEST.size
			self.buttonColumns=REQUEST.buttonColumns

			self.zeroform=REQUEST.zeroform
			self.zerodata=REQUEST.zerodata

			self.required=REQUEST.required

			self.ismenu=0
			if REQUEST.has_key('ismenu'):
				self.ismenu=1
			
			self.multiple=0		
			if REQUEST.has_key('multiple'):
				self.multiple=1
				
			self.providezero=0    
			if REQUEST.has_key('providezero'):
				self.providezero=1
			
			self.allowzero=0
			if REQUEST.has_key('allowzero'):
				self.allowzero=1

	def __getModifyer__(self):
		    if self.datatype=="int":
			return int
		    else:
			return str
		    
	def __str__(self):
		return self.index_html(self)

	def __call__(self):
		return self.index_html(self)

	def renderMenu(self,REQUEST):
		datatype=''
		multiple=''

		if self.datatype!='':
			datatype=':'+self.datatype
			
		if self.multiple==1:
			datatype=':list'+datatype
			multiple='multiple'
			
		data=self.aq_acquire(self.source)(REQUEST=REQUEST)
		selectedList=self.value()			

		htmltext='<select name="%s%s" %s size="%s">\n' \
			    % (self.id, datatype, multiple, self.size)
			    
		if self.providezero and (not len(selectedList) > 0 \
		    or self.allowzero or REQUEST.has_key('error_info')):
			zeroselected=''
			if (self.__getModifyer__()(self.zeroform) in \
				selectedList) or len(data)==0:
			    zeroselected='selected'
			htmltext=htmltext+'<option value="%s" %s>%s\n' \
				% (self.zeroform, zeroselected, self.hint)
				
		for valuepair in data:
			selected=''
			if valuepair['value'] in selectedList:
				selected='selected'
			htmltext=htmltext+'<option value="%s" %s> %s\n' \
			    %(valuepair['value'], selected, valuepair['title'])
		htmltext=htmltext+'</select>' 
		return htmltext

	def renderButtons(self, REQUEST):
		datatype=''
		widget='radio'
		htmltext=''
		buttonNo=0
		
		if self.datatype!='':
			datatype=':'+self.datatype
		
		if self.multiple==1:
			datatype=':list'+datatype
			widget='checkbox'
			
		data=self.aq_acquire(self.source)(REQUEST=REQUEST)
		selectedList=self.value()			

		htmltext='<table><tr>'
		if self.providezero and (not len(selectedList) > 0 \
		    or self.allowzero or REQUEST.has_key('error_info')):
		        buttonNo=buttonNo+1
			zeroselected=''
			if (self.__getModifyer__()(self.zeroform) in \
				selectedList) or len(data)==0:
			    zeroselected='checked'
			htmltext=htmltext+'<td><input type="%s" name="%s%s" ' \
					    'value="%s" %s></td><td>%s</td>\n' \
				% (widget, self.id, datatype, self.zeroform, \
				    zeroselected, self.hint)

		for valuepair in data:
			if (buttonNo%self.buttonColumns==0):
			    htmltext=htmltext+'</tr><tr>'
			buttonNo=buttonNo+1
			selected=''
			if valuepair['value'] in selectedList:
				selected='checked'
			htmltext=htmltext+'<td><input type="%s" name="%s%s" ' \
					    'value="%s" %s></td><td> %s</td>\n' \
			    %(widget, self.id, datatype, valuepair['value'], \
				selected, valuepair['title'])
		return htmltext+'</tr></table>'
		

	def __validate__(self, against=None):
		if against is not None:
			if type(against)==types.ListType:
				against_=against
			else:
			    against_=[against,]

			if self.multiple:
			    requiredCheck=self.required
			else:
			    requiredCheck=1
			if len(against_) < (requiredCheck):
			    return False
			
			if (self.providezero==1) and \
			   (self.__getModifyer__()(self.zeroform) in against_):
				if self.allowzero:
				    position=against_.index( \
					self.__getModifyer__()(self.zeroform))
				    r_env = rexec.RExec()
				    against_[position]= \
					r_env.r_eval(self.zerodata)
				else:
				    return False				
		return True

	def height(self):
	    if self.ismenu:
		height=int(self.size)-1
		if height < 1:
			height=1
	    else:
		data=self.aq_acquire(self.source)(REQUEST=self.REQUEST)
		height=int(len(data) / self.buttonColumns)
		if (len(data) % self.buttonColumns) > 1:
		    height=height+1
	    return height

	def value(self):
		selectedList=[]
		selected=[]
		if len(self.default)>0:
		    try:
			selected=[self.__getModifyer__()(sel) for sel in \
				    self.default.split(',')]
		    except:
			pass
		
		try:
		    selected=self.REQUEST['__source_data__'][self.id]
		except:
		    pass
		    
		try:
		    selected=self.REQUEST[self.id]
		except:
		    pass
		if type(selected)==types.ListType:
			selectedList.extend(selected)
		else:
			selectedList.append(selected)
		return selectedList
	

# $Log: HTMLMenu.py,v $
# Revision 1.17  2001/06/27 10:14:06  ctheune
# Bug # 124: ZSV Data Conversion, the real value is beeing executed in an
# jail, for having the possibility to add values like 'None' to
# a database or to have lightweight simple computations ...
#
# Revision 1.16  2001/06/12 19:39:24  ctheune
# fixed typo (multpiple)
# readded preselection from initial form data
# fixed typecasted handling of ZSV
#
# Revision 1.15  2001/06/11 22:17:45  ctheune
# added new changeProperties behaviour for editing Items
# cleaned up
# added support for new spanning Method
# added ButtonView (radioButton/Checkbox) as alternative to menu
# corrected height() to work with new view mode
#
# Revision 1.14  2001/06/07 16:26:27  ctheune
# fixed bug 118
#
# Revision 1.13  2001/06/05 07:34:22  ctheune
# added standard value for "required"
# fixed a little bug in the value() function
#
# Revision 1.12  2001/06/04 09:47:36  ctheune
#  fixed bug #39
#  fixed bug #91
#  cleaned up source code & reformatting
#  wrote better comments
#
# Revision 1.11  2001/06/02 17:01:01  ctheune
# fixed concurrency problems, that occurred, by modifying 2 versions on 1 desk. sorry
#
# Revision 1.10  2001/05/29 09:12:09  zagy
# Fixex 3 syntax errors
#
# Revision 1.9  2001/05/27 10:26:55  ctheune
# fixed bug #50
# fixed bug #61
#
# Revision 1.8  2001/04/12 20:47:42  ctheune
# fixed stability leak on validation check
#
# Revision 1.7  2001/04/12 10:28:38  ctheune
# fixed bug #14
#
# Revision 1.6  2001/04/10 20:02:41  ctheune
# fixed bug #12
#
# Revision 1.5  2001/04/04 15:59:58  ctheune
# Fixed BUG #13
#
# Revision 1.4  2001/04/04 15:20:16  ctheune
# Fixed BUG #1
#
# Revision 1.3  2001/04/03 17:45:09  ctheune
# changed cvs tag Revision to ID as described in Bug #10 on bugzilla.gocept.com
#
# Revision 1.2  2001/04/01 14:40:28  ctheune
# cleaned up code, fixed BUG #1
#
# Revision 1.1  2001/03/30 12:46:45  ctheune
# added HTML menu class
#
