# Formulator - Let's you manage Forms on the ZOPE Application Server
# Copyright (C) 2001 Christian Theune, gocept gmbh & co. kg
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Module for Textarea-Widgets
#
# $Id: HTMLTextarea.py,v 1.8 2001/06/11 22:18:40 ctheune Exp $

__doc__="""HTMLWidget Textarea module."""
__version__='0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
from OFS.DTMLDocument import DTMLDocument
import Acquisition
import AccessControl

import re

True=1
False=0

manage_addHTMLTextareaForm=HTMLFile('dtml/HTMLTextareaAdd', globals())

def manage_addHTMLTextarea(self, id, REQUEST=None):
    """Adds a new HTML Textarea Widget."""
    self._setObject(id, HTMLTextarea(id, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class HTMLTextarea(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit,
                	AccessControl.Role.RoleManager):
    """HTMLWidget for Textareas"""

    meta_type='HTML Textarea'
    css_class='textarea'
    
    colspan=0
    wrapping='virtual'
    
    manage_properties=HTMLFile('dtml/HTMLTextareaEdit', globals())
    
    manage_options=(
        {'label':'Properties', 'action':'manage_properties', 'target':None},
        {'label':'Security', 'action':'manage_access', 'target':None},
        )

    def __init__(self, id, REQUEST=None):
        self.id=id
	if REQUEST is not None:
	    self.changeProperties(REQUEST)

    def index_html(self, REQUEST=None):
        return '<textarea name="%s" cols="%s" rows="%s" wrapping="%s">' \
		'%s</textarea>' % (self.id, self.cols, self.rows, \
		self.wrapping, self.value(REQUEST))

    def manage_edit(self, REQUEST=None):
        """Sets the new properties."""
	if REQUEST is not None:
	    self.changeProperties(REQUEST)
            return MessageDialog(title='Edited',
                message="Properties for %s changed." % self.id,
                action='./manage_properties')

    def changeProperties(self, REQUEST):
        if REQUEST is not None:
            self.default=REQUEST.default
            self.regexp=REQUEST.regexp
            self.title=REQUEST.title
            self.hint=REQUEST.hint
            self.cols=REQUEST.cols
            self.rows=REQUEST.rows
	    
            self.column=REQUEST.column
            self.row=REQUEST.row
	    self.colspan=REQUEST.colspan

	    if REQUEST.has_key("wrapping"):
		self.wrapping=REQUEST["wrapping"]
	    
	
    def __str__(self):
        return self.index_html(self)

    def __call__(self):
        return self.index_html(self)
    
    def __validate__(self, against=None):
        recomp = re.compile(self.regexp, re.DOTALL | re.IGNORECASE)
        if len(recomp.findall(against)):
            return True
        return False

    def height(self):
        return self.rows-2

    def value(self, REQUEST=None):
	try:
            return self.REQUEST[self.id]
	except KeyError:
            pass

	try:
            return self.REQUEST['__source_data__'][self.id]
	except KeyError:
	    pass

        return self.default
    

# $Log: HTMLTextarea.py,v $
# Revision 1.8  2001/06/11 22:18:40  ctheune
# added new changeProperties behaviour for editing Items
# cleaned up
# added support for new spanning Method
#
# Revision 1.7  2001/06/07 16:26:27  ctheune
# fixed bug 118
#
# Revision 1.6  2001/06/07 16:47:39  ctheune
# made standard wrap as 'virtual'
#
# Revision 1.5  2001/06/05 07:31:57  ctheune
# fixed bug #65
#
# Revision 1.4  2001/06/02 17:01:01  ctheune
# fixed concurrency problems, that occurred, by modifying 2 versions on 1 desk. sorry
#
# Revision 1.3  2001/04/12 20:48:33  ctheune
# removed isrequired field. depricated, use regular expressions instead.
#
# Revision 1.2  2001/04/03 17:45:09  ctheune
# changed cvs tag Revision to ID as described in Bug #10 on bugzilla.gocept.com
#
# Revision 1.1  2001/03/30 12:46:59  ctheune
# added HTML textarea class
#
