# Formulator - Let's you manage Forms on the ZOPE Application Server
# Copyright (C) 2001 Christian Theune, gocept gmbh & co. kg
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Module for Textfield-Widgets
#
# $Id: HTMLTextfield.py,v 1.6 2001/06/11 22:19:52 ctheune Exp $

__doc__="""HTMLWidget Textfield module."""
__version__='0.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
from OFS.DTMLDocument import DTMLDocument
import Acquisition
import AccessControl

import re

True=1
False=0

manage_addHTMLTextfieldForm=HTMLFile('dtml/HTMLTextfieldAdd', globals())

def manage_addHTMLTextfield(self, id, REQUEST=None):
    """Adds a new HTML textfield."""
    
    self._setObject(id, HTMLTextfield(id, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class HTMLTextfield(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit,
                AccessControl.Role.RoleManager):
    """HTMLWidget for Textfields"""

    meta_type='HTML Textfield'
    css_class=None
    colspan=0
       
    manage_properties=HTMLFile('dtml/HTMLTextfieldEdit', globals())
    
    manage_options=(
        {'label':'Properties', 'action':'manage_properties', 'target':None},
        {'label':'Security', 'action':'manage_access', 'target':None},
        )

    def __init__(self, id, title, default, size, maxlength, regexp, \
			ispassword, hint, column, row):
        self.id=id
	if REQUEST is not None:
	    self.changeProperties(REQUEST)
	    
    def index_html(self, REQUEST=None):
        if (self.ispassword==1) or (self.ispassword=="1"):	# string has migration status
            type='password'
        else:
            type='text'

        return '<input type="%s" name="%s" value="%s" size="%s" ' \
		'maxlength="%s">' % (type, self.id, self.value(REQUEST), \
		self.size,self.maxlength)

    def manage_edit(self, REQUEST=None):
        """ set the new properties """
	if REQUEST is not None:
	    self.changeProperties(REQUEST)
            return MessageDialog(title='Edited',
                message="Properties for %s changed." % self.id,
                action='./manage_properties')

    def changeProperties(self, REQUEST=None):
        if REQUEST is not None:
            self.default=REQUEST.default
            self.maxlength=REQUEST.maxlength
            self.regexp=REQUEST.regexp
            self.size=REQUEST.size
            self.title=REQUEST.title
            self.hint=REQUEST.hint
            self.column=REQUEST.column
            self.row=REQUEST.row
	    self.colspan=REQUEST.colspan
	    self.ispassword=0
            if REQUEST.has_key('ispassword'):
                self.ispassword=1   

    def __str__(self):
        return self.index_html(self)

    def __call__(self):
        return self.index_html(self)

    def __validate__(self, against=None):
        recomp=re.compile(self.regexp, re.DOTALL | re.IGNORECASE)
        if len(recomp.findall(against or '')):
            return True
        return False

    def height(self):
        return 1

    def value(self, REQUEST=None):
	ret=self.default

	try:
	    	ret=self.REQUEST[self.id]
	except KeyError:
		pass

	try:
		ret=self.REQUEST['__source_data__'][self.id]
	except KeyError:
		pass

	return (ret or "")
    

# $Log: HTMLTextfield.py,v $
# Revision 1.6  2001/06/11 22:19:52  ctheune
# added new changeProperties behaviour for editing Items
# cleaned up
# added support for new spanning Method
# minor (silent) interface changes, to have not everything as string but int
# this was to have the ispassword as int ... migration is granted
#
# Revision 1.5  2001/06/07 16:26:27  ctheune
# fixed bug 118
#
# Revision 1.4  2001/04/12 20:48:50  ctheune
# removed isrequired field. depricated, use regular expressions instead.
#
# Revision 1.3  2001/04/10 20:03:55  ctheune
# little cleaning
#
# Revision 1.2  2001/04/03 17:45:09  ctheune
# changed cvs tag Revision to ID as described in Bug #10 on bugzilla.gocept.com
#
# Revision 1.1  2001/03/30 12:48:12  ctheune
# added HTML textfield class
#
